@@ -15,19 +15,15 @@
 # AccessControl.ZopeGuards contains a large set of wrappers for builtins.
 # DocumentTemplate.DT_UTil contains a few.
 
-from RestrictedPython import _compat
+import builtins
 
+from RestrictedPython._compat import IS_PY311_OR_GREATER
 
-if _compat.IS_PY2:  # pragma: PY2
-    import __builtin__ as builtins
-else:  # pragma: PY3
-    # Do not attempt to use this package on Python2.7 as there
-    # might be backports for this package such as future.
-    import builtins
 
 safe_builtins = {}
 
 _safe_names = [
+    '__build_class__',
     'None',
     'False',
     'True',
@@ -109,22 +105,8 @@ _safe_exceptions = [
     'ZeroDivisionError',
 ]
 
-if _compat.IS_PY2:  # pragma: PY2
-    _safe_names.extend([
-        'basestring',
-        'cmp',
-        'long',
-        'unichr',
-        'unicode',
-        'xrange',
-    ])
-    _safe_exceptions.extend([
-        'StandardError',
-    ])
-else:  # pragma: PY3
-    _safe_names.extend([
-        '__build_class__',  # needed to define new classes
-    ])
+if IS_PY311_OR_GREATER:
+    _safe_exceptions.append("ExceptionGroup")
 
 for name in _safe_names:
     safe_builtins[name] = getattr(builtins, name)
@@ -202,7 +184,7 @@ def _write_wrapper():
             f(*args)
         return handler
 
-    class Wrapper(object):
+    class Wrapper:
         def __init__(self, ob):
             self.__dict__['ob'] = ob
 
@@ -264,12 +246,9 @@ def safer_getattr(object, name, default=None, getattr=getattr):
     http://lucumr.pocoo.org/2016/12/29/careful-with-str-format/
 
     """
-    if name in ('format', 'format_map') and (
-            isinstance(object, _compat.basestring) or (
-            isinstance(object, type)
-            and issubclass(object, _compat.basestring))):
+    if isinstance(object, str) and name == 'format':
         raise NotImplementedError(
-            'Using the string format* methods is not safe')
+            'Using format() on a %s is not safe.' % object.__class__.__name__)
     if name.startswith('_'):
         raise AttributeError(
             '"{name}" is an invalid attribute name because it '
