@@ -1,17 +1,17 @@
+import platform
+import types
+
+import pytest
+
+from RestrictedPython import CompileResult
 from RestrictedPython import compile_restricted
 from RestrictedPython import compile_restricted_eval
 from RestrictedPython import compile_restricted_exec
 from RestrictedPython import compile_restricted_single
-from RestrictedPython import CompileResult
-from RestrictedPython._compat import IS_PY2
-from RestrictedPython._compat import IS_PY3
 from RestrictedPython._compat import IS_PY38_OR_GREATER
+from RestrictedPython._compat import IS_PY310_OR_GREATER
 from tests.helper import restricted_eval
 
-import platform
-import pytest
-import types
-
 
 def test_compile__compile_restricted_invalid_code_input():
     with pytest.raises(TypeError):
@@ -40,7 +40,9 @@ INVALID_ASSINGMENT = """
 def test_compile__invalid_syntax():
     with pytest.raises(SyntaxError) as err:
         compile_restricted(INVALID_ASSINGMENT, '<string>', 'exec')
-    if IS_PY38_OR_GREATER:
+    if IS_PY310_OR_GREATER:
+        assert "SyntaxError: cannot assign to literal here." in str(err.value)
+    elif IS_PY38_OR_GREATER:
         assert "cannot assign to literal at statement:" in str(err.value)
     else:
         assert "can't assign to literal at statement:" in str(err.value)
@@ -100,12 +102,7 @@ def test_compile__compile_restricted_exec__5():
     assert result.code is None
     assert result.warnings == []
     assert result.used_names == {}
-    if IS_PY2:
-        assert result.errors == (
-            'compile() expected string without null bytes',)
-    else:
-        assert result.errors == (
-            'source code string cannot contain null bytes',)
+    assert result.errors == ('source code string cannot contain null bytes',)
 
 
 EXEC_STATEMENT = """\
@@ -114,15 +111,18 @@ def no_exec():
 """
 
 
-@pytest.mark.skipif(
-    IS_PY2,
-    reason="exec statement in Python 2 is handled by RestrictedPython ")
 def test_compile__compile_restricted_exec__10():
-    """It is a SyntaxError to use the `exec` statement. (Python 3 only)"""
+    """It is a SyntaxError to use the `exec` statement."""
     result = compile_restricted_exec(EXEC_STATEMENT)
-    assert (
-        'Line 2: SyntaxError: Missing parentheses in call to \'exec\' at '
-        'statement: "exec \'q = 1\'"',) == result.errors
+    if IS_PY310_OR_GREATER:
+        assert (
+            'Line 2: SyntaxError: Missing parentheses in call to \'exec\'. Did'
+            ' you mean exec(...)? at statement: "exec \'q = 1\'"',
+        ) == result.errors
+    else:
+        assert (
+            'Line 2: SyntaxError: Missing parentheses in call to \'exec\' at'
+            ' statement: "exec \'q = 1\'"',) == result.errors
 
 
 FUNCTION_DEF = """\
@@ -164,24 +164,18 @@ def test_compile__compile_restricted_csingle():
 
 PRINT_EXAMPLE = """
 def a():
-    print 'Hello World!'
+    print('Hello World!')
 """
 
 
-@pytest.mark.skipif(
-    IS_PY3,
-    reason="Print statement is gone in Python 3."
-           "Test Deprecation Warming in Python 2")
 def test_compile_restricted():
-    """This test checks compile_restricted itself if that emit Python warnings.
+    """It emits Python warnings.
+
     For actual tests for print statement see: test_print_stmt.py
     """
     with pytest.warns(SyntaxWarning) as record:
         result = compile_restricted(PRINT_EXAMPLE, '<string>', 'exec')
         assert isinstance(result, types.CodeType)
-        # Non-CPython versions have a RuntimeWarning, too.
-        if len(record) > 2:  # pragma: no cover
-            record.pop()
         assert len(record) == 1
         assert record[0].message.args[0] == \
             "Line 2: Prints, but never reads 'printed' variable."
